import os
class Writer:
    """
    Write parser result into python file
    """
    def __init__(self, directory):
        self.directory = directory
        os.mkdir(self.directory)

    def create(self, content):
        for fname, code in content.items():
            with open(os.path.join(self.directory, fname), 'w') as f:
                f.write(code)




from unixsocketcs.client_cli import CLI

class UI(CLI):
    def __init__(self, prog="{{ client_prog_name }}"):
        super(UI, self).__init__(prog)
        subparser = self.parser.add_subparsers(title='commands', dest='cmd', help='commands help')




import argparse
from .confparser import Parser
from .codegen import CodeGener
from .appwriter import Writer

class CLI:
    def __init__(self, prog="Demo program"):
        parser = argparse.ArgumentParser(description=prog)
        parser.add_argument('--conf', dest='conf')
        parser.add_argument('--app', dest='app')
        self.parser = parser

    def parse(self):
        args = vars(self.parser.parse_args())
        return args

if __name__ == "__main__":
    args = CLI().parse()
    cfg, dst = args["conf"], args["app"]
    info = Parser(cfg).info
    codeinfo = CodeGener(info).genCode()
    Writer(dst).create(codeinfo)




from unixsocketcs.server_handler import Handler
import json

class Logic(Handler):
    def process(self, data):
        order = json.loads(data)
        cmd = order['cmd'] 
        if cmd == None:
            cmd = '_default'
        action = getattr(self, cmd)
        return action(order['args'])

    
    def _formdata(self, msg):
        return bytes([len(msg)]) + bytes(msg, 'utf-8')

    def _default(self, args):
        message = 'no cmd is specified'
        return self._formdata(message)




import json
from jinja2 import Environment, Template, FileSystemLoader

class Parser:
    """
    parse conf 
    """
    def __init__(self, conf):
        self.conf, self.info = conf, {}
        self.parse()

    def parse(self):
        with open(self.conf, 'r') as f:
            self.info = json.load(f)
            if 'server_address' not in self.info.keys() or self.info['server_address'] == '':
                self.info['server_address'] = './uds_server'
            if 'client_prog_name' not in self.info.keys() or self.info['client_prog_name'] == '':
                self.info['client_prog_name'] = 'uds client'
            if 'features' not in self.info.keys() or not isinstance(self.info['features'], list):
                self.info['features'] = []



import json
from jinja2 import Environment, Template, FileSystemLoader
import os

class CodeGener:
    """
    Init templates 
    """
    def __init__(self, info):
        self.info = info
        tmpldir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'templates')
        env = Environment(
            loader=FileSystemLoader(searchpath=tmpldir)
        )

        self.tmpl = {}
        self.tmpl['app_client'] = env.get_template('tmpl_app_client.py')
        self.tmpl['app_server'] = env.get_template('tmpl_app_server.py')
        self.tmpl['handler'] = env.get_template('tmpl_handler.py')
        self.tmpl['handler_fn'] = env.get_template('tmpl_handler_fn.py')
        self.tmpl['ui'] = env.get_template('tmpl_ui.py')
        self.tmpl['ui_subparser'] = env.get_template('tmpl_ui_subparser.py')

        self.codeinfo = {}


    def gen_app_client(self):
        code = self.tmpl['app_client'].render(server_address=self.info['server_address']) + '\n'
        self.codeinfo['app_client.py'] = code

    def gen_app_server(self):
        code = self.tmpl['app_server'].render(server_address=self.info['server_address']) + '\n'
        self.codeinfo['app_server.py'] = code

    def _gen_ui_subparser(self):
        subparser_code = '\n'.join([self.tmpl['ui_subparser'].render(cmd=cmd) for cmd in self.info['features']]) + '\n'
        return subparser_code

    def gen_ui(self):
        ui_code = self.tmpl['ui'].render(client_prog_name=self.info['client_prog_name']) + '\n'
        subparser_code = self._gen_ui_subparser()
        code = '\n'.join([ui_code, subparser_code]) + '\n' 
        self.codeinfo['ui.py'] = code

    def _gen_handler_fn(self):
        handler_fn_code = '\n'.join([self.tmpl['handler_fn'].render(action=cmd) for cmd in self.info['features']]) + '\n'
        return handler_fn_code 

    def gen_handler(self):
        handler_code = self.tmpl['handler'].render() + '\n'
        handler_fn_code = self._gen_handler_fn()
        code = '\n'.join([handler_code, handler_fn_code]) + '\n' 
        self.codeinfo['handler.py'] = code


    def genCode(self):
        self.gen_app_client()
        self.gen_app_server()
        self.gen_ui()
        self.gen_handler()
        return self.codeinfo




import setuptools
V="0.0.1"
with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cg-unixsocketcs",
    version=V,
    author="phoenix.lv",
    author_email="phoenix.grey0108@gmail.com",
    description="code generator to create unix domain socket server and client",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hellstein/codegen-unixsocketcs",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
